"""Saku Admin CGI methods.
"""
#
# Copyright (c) 2005,2006 shinGETsu Project.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: admin_cgi.py 1161 2006-03-07 15:07:03Z fuktommy $
#

import os
import cgi
import re
import md5
from random import random

import config
import gateway
from cache import *
from node import *

__version__ = "$Revision: 1161 $"


class CGI(gateway.CGI):

    """Class for /admin.cgi."""

    def run(self):
        path = self.path_info()
        form = cgi.FieldStorage(environ=self.environ, fp=self.stdin)

        cmd = form.getfirst('cmd', '')
        if not self.isadmin:
            self.print403()
            return
        elif (cmd == 'rdel') or (cmd == 'fdel'):
            rm_files = form.getlist('file')
            rm_records = form.getlist('record')
            if (not rm_files) or ((cmd == 'rdel') and (not rm_records)):
                self.print404()
            elif cmd == 'fdel':
                self.print_delete_file(rm_files)
            else:
                self.print_delete_record(rm_files[0], rm_records)
        elif ((cmd == 'xrdel') or (cmd == 'xfdel')) and \
             self.environ["REQUEST_METHOD"] == "POST" and \
             self.check_sid(form.getfirst("sid", "")):
            rm_files = form.getlist('file')
            rm_records = form.getlist('record')
            if (not rm_files) or ((cmd == 'xrdel') and (not rm_records)):
                self.print404()
            elif cmd == 'xfdel':
                self.do_delete_file(rm_files)
            else:
                self.do_delete_record(rm_files[0],
                                      rm_records,
                                      form.getfirst("dopost", ""),
                                      form)
        elif path == "status":
            self.print_status()
        elif path.startswith("search"):
            self.print_search(path, form)
        else:
            self.print404()

    def make_sid(self):
        """Make admin sid for security."""
        sidfile = config.admin_sid
        r = ""
        for i in range(4):
            r += str(random())
        sid = md5.new(r).hexdigest()
        try:
            file(sidfile, "wb").write(sid + "\n")
        except IOError, err:
            self.stderr.write("%s: IOError: %s\n" % (sidfile, err))
        return sid

    def check_sid(self, sid):
        """Check admin sid for security."""
        sidfile = config.admin_sid
        try:
            saved = file(sidfile).read().strip()
            os.remove(sidfile)
            return sid == saved
        except IOError, err:
            self.stderr.write("%s: IOError: %s\n" % (sidfile, err))
            return False
        except OSError, err:
            self.stderr.write("%s: OSError: %s\n" % (sidfile, err))
            return sid == saved

    def print_delete_record(self, datfile, records):
        """Delete record dialog."""
        sid = self.make_sid()
	self.header(self.message["del_record"], deny_robot=True)
        self.stdout.write(
            "<p>" + self.message["del_record_q"] + "</p>\n" +
            '<form method="post" action="' + self.admin_cgi + '"><p>\n' +
            '  <input type="hidden" name="cmd" value="xrdel" />\n' +
            '  <input type="hidden" name="file" value="%s" />\n' %datfile +
            '  <input type="hidden" name="sid" value="%s" />\n' % sid +
            '  <input type="submit" name="submit" value="%s"' %
                self.message["yes"] +
            ' tabindex="1" accesskey="w" />\n')
        if len(records) == 1:
            self.stdout.write(
                '  <input type="checkbox" name="dopost" value="dopost"' +
                ' tabindex="2" accesskey="s" />' + 
                self.message["send"] + "<br />\n" +
                "  " + self.message["name"] +
                ':<input name="name" size="15" value=""' +
                ' tabindex="3" accesskey="n" />\n' +
                "  " + self.message["signature"] +
                ':<input type="password" name="passwd" size="15"' +
                ' value="" tabindex="4" accesskey="p" />\n' +
                "  " + self.message["comment"] + 
                ':<input name="body" size="15" value=""' +
                ' tabindex="5" accesskey="m" title="%s" />\n' %
                self.message['desc_comment'])
        self.stdout.write('</p>\n')
        for r in records:
            rec = Record(datfile=datfile, idstr=r)
            if rec.exists():
                self.print_delete_record_form(datfile, rec)
            else:
                self.stdout.write("<p>" + self.message["no_record"] + "</p>\n")
        self.stdout.write("</form>\n")
        self.footer()

    def print_delete_record_form(self, datfile, rec):
        rec.load()
        buf = re.sub(r"attach:[^<>]+", "attach:1", rec.recstr)
        buf = cgi.escape(buf)
        self.stdout.write(
            '  <p><input type="checkbox" checked="checked"' +
            ' tabindex="1" accesskey="f"' +
            ' name="record" value="%s" />' % rec.idstr + buf + "</p>\n")

    def do_delete_record(self, datfile, records, dopost="", form=None):
        for type in config.types:
            title = self.str_encode(self.file_decode(datfile))
            if datfile.startswith(type + "_"):
                next = self.appli[type] + self.sep + title
                break
        else:
            next = self.root
        for r in records:
            rec = Record(datfile=datfile, idstr=r)
            size = rec.size()
            removed = rec.remove()
            cache = Cache(datfile)
            cache.sync_body_light(size=size)
            if removed and (dopost != ""):
                self.post_delete_message(cache, rec, form)
                break
        self.print302(next)

    def post_delete_message(self, cache, rec, form):
        """Post delete message to other nodes."""
        stamp = self.error_time()

        body = {}
        for key in ("name", "body"):
            if form.getfirst(key, "") != "":
                body[key] = self.escape(form.getfirst(key, ""))
        body["remove_stamp"] = str(rec.stamp)
        body["remove_id"] = rec.id
        passwd = form.getfirst("passwd", "")
        id = rec.build(stamp, body, passwd=passwd)
        cache.add_data([rec])

        updatelist = UpdateList()
        updatelist.append(rec)
        updatelist.sync()
        recentlist = RecentList()
        recentlist.append(rec)
        recentlist.sync()
        nodelist = NodeList()
        nodelist.tell_update(cache, stamp=stamp, id=id)

    def print_delete_file(self, files):
        """Delete file dialog."""
        sid = self.make_sid()
	self.header(self.message["del_file"], deny_robot=True)
        self.stdout.write(
            '<form method="post" action="' + self.admin_cgi + '"><p>\n' +
            "  " + self.message["del_file_q"] + "\n" +
            '  <input type="hidden" name="cmd" value="xfdel" />\n' +
            '  <input type="hidden" name="sid" value="%s" />\n' % sid +
            '  <input type="submit" value="%s" tabindex="1" accesskey="y" />\n'
            % self.message["yes"] + '</p>\n')
        for c in files:
            cache = Cache(c)
            if cache.exists():
                self.print_delete_file_form(cache)
            else:
                self.stdout.write("<p>%s: %s</p>\n" %
                                  (c, self.message["no_file"]))
        self.stdout.write('</form>\n')
        self.footer()

    def print_delete_file_form(self, cache):
        for type in config.types:
            if cache.datfile.startswith(type + "_"):
                title = self.file_decode(cache.datfile)
                break
        else:
            title = cache.datfile
        self.stdout.write(
            '<h2><input type="checkbox" checked="checked"' +
            ' tabindex="1" accesskey="f"' +
            ' name="file" value="%s" />%s</h2>\n' % (cache.datfile, title))
        count = 0
        for idstr in cache:
            count += 1
            rec = cache[idstr]
            rec.load()
            buf = re.sub(r"attach:[^<>]+", "attach:1", rec.recstr)
            buf = cgi.escape(buf)
            self.stdout.write("<p>%s</p>\n" % buf)
            if (count > 2):
                break

    def do_delete_file(self, files):
        for c in files:
            cache = Cache(c)
            cache.remove()
        self.print302(self.gateway_cgi + self.sep + "changes")

    def print_search_form(self, query=""):
        self.stdout.write(
            '<form method="get" action="' + self.admin_cgi + '/search"><p>\n'+
            '  <input name="query" size="19" value="' + query + '"' +
            ' tabindex="1" accesskey="q" />\n' +
            '  <input type="submit" value="' + self.message["search"] + '"' +
            ' name="submit" tabindex="1" accesskey="s" />\n' +
            '</p></form>\n')

        self.stdout.write(
            '<form method="get" action="' + config.google + '"><p>\n' +
            '  <input name="q" size="19" value="' + query + '"'
            ' tabindex="1" accesskey="g" />\n' +
            '  <input type="hidden" name="ie" value="UTF-8" />\n' +
            '  <input type="submit" name="google" value="' +
            self.message["google"] + '" tabindex="1" accesskey="g" />\n' +
            "</p></form>\n")

    def print_search_result(self, query):
        self.header(self.message["search"], deny_robot=True)
        self.stdout.write('<p>%s</p>\n' % self.message['desc_search'])
        cachelist = CacheList().search(query)
        self.print_index_list(cachelist, footer=False)
        self.print_search_form(query)
        self.footer()

    def print_search(self, path, form):
        query = cgi.escape(form.getfirst("query", ""), True)
        if query == "":
            query = path[len("search/"):].replace('"', "&quote;")

        if query == "":
            query = self.environ.get("QUERY_STRING", "")
            query = self.escape(self.str_decode(query))

        if query == "":
            self.header(self.message["search"], deny_robot=True)
            self.stdout.write('<p>%s</p>\n' % self.message['desc_search'])
            self.print_search_form()
            self.footer()
        else:
            self.print_search_result(query)

    def print_status(self):
        self.header(self.message["status"], deny_robot=True)
        self.stdout.write('<table summary="%s">\n' % self.message["status"])

        nodelist = NodeList()
        searchlist = SearchList()
        cachelist = CacheList()

        records = 0
        size = 0
        for cache in cachelist:
            records += len(cache)
            size += cache.size

        myself = nodelist.myself()

        for p in (("linked_nodes", len(nodelist)),
                  ("known_nodes", len(searchlist)),
                  ("files", len(cachelist)),
                  ("records", records),
                  ("cache_size", str(2*size*10/1024/1024/10.0) +
                                 self.message["mb"]),
                  ("self_node", myself)):
            self.stdout.write("  <tr><td>%s</td><td>%s</td></tr>\n" %
                              (self.message[p[0]], str(p[1])))

        self.stdout.write("</table>")

        for p in (("linked_nodes", nodelist),
                  ("known_nodes", searchlist)):
            self.stdout.write("<h2>%s</h2>\n<ul>\n" % self.message[p[0]])
            for node in p[1]:
                self.stdout.write("  <li>%s</li>\n" % str(node))
            self.stdout.write("</ul>\n")

        self.footer()

# End of CGI
