'''Spam Rules List.
'''
#
# Copyright (c) 2006 shinGETsu Project.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: $
#

import re
import sys
import os.path

import config

__version__ = '$Revision: $'
__all__ = ['rules']


class SpamRules:
    '''Spam Rules List.
    '''
    rules = []

    def __init__(self):
        self.rules = []
        try:
            f = file(config.spam_list)
            for line in f:
                pat = line.strip()
                if pat:
                    self.rules.append(re.compile(pat))
            f.close()
        except (IOError, OSError), err:
            sys.stderr.write('IOError/OSError: %s: %s\n' %
                             (config.spam_list, err))

    def check(self, recstr):
        '''Check record as string.

        Check it is a spam or not.
        '''
        for r in self.rules:
            if r.search(recstr):
                return True
        return False

# End of SpamRules


_cached_rules = SpamRules()
_mtime = os.path.getmtime(config.spam_list)

def rules():
    global _cached_rules
    global _mtime
    mtime = os.path.getmtime(config.spam_list)
    if mtime < _mtime:
        _mtime = mtime
        _cached_rules = SpamRules()
        return _cached_rules
    else:
        return _cached_rules
